@@ -1,5 +1,5 @@
 package ZMQ::Raw::Socket;
-$ZMQ::Raw::Socket::VERSION = '0.07';
+$ZMQ::Raw::Socket::VERSION = '0.08';
 use strict;
 use warnings;
 use Carp;
@@ -31,7 +31,7 @@ ZMQ::Raw::Socket - ZeroMQ Socket class
 
 =head1 VERSION
 
-version 0.07
+version 0.08
 
 =head1 DESCRIPTION
 
@@ -47,6 +47,17 @@ A L<ZMQ::Raw::Socket> represents a ZeroMQ socket.
 	# receive all message parts
 	my @msgs = $socket->recvmsg();
 
+	# send multiple message parts
+	$socket->sendmsg ('hello', 'world'); # flags cannot be used here
+
+	# or
+	my $msg1 = ZMQ::Raw::Message->new;
+	$msg1->data ('hello');
+
+	my $msg2 = ZMQ::Raw::Message->new;
+	$msg2->data ('world');
+	$socket->sendmsg ($msg1, $msgs2, 0); # flags can be used here
+
 =head1 METHODS
 
 =head2 new( $context, $type )
@@ -134,14 +145,20 @@ follow.
 
 =head2 sendmsg( @msgs, $flags = 0)
 
-Queue C<@msgs> to be sent.
+Queue C<@msgs> to be sent. Each message in C<@msgs> that is a L<C<ZMQ::Raw::Message>>
+is still valid after this call, that is, they may be reused. Each item in C<@msgs>
+may either be a L<C<ZMQ::Raw::Message>> object or a "normal" perl scalar. The
+C<$flags> parameter is only available if all items in C<@msgs> are L<C<ZMQ::Raw::Message>>
+objects. See the SYNOPSIS for usage examples.
 
 =head2 recv( $flags = 0)
 
 Receive a message. If there are no messages available the method will block
 until the request can be satisfied unless the C<ZMQ_DONTWAIT> flag is specified.
 If a message is not available and C<ZMQ_DONTWAIT> has been specified, this
-method will return C<undef> immediately.
+method will return C<undef> immediately. If called in list context, this method
+will return each part of the message as a scalar item. In scalar context, each
+part of the message will be concatenated into a single scalar item.
 
 =head2 recvmsg( $flags = 0)
 
